define([
    'jquery',
    'underscore',
    'backbone',
    'text!tmpl/drop_view.html',
    'multi_upload_settings'
],
function($, _, Backbone, tmpl, MultiUploadSettings) {
    'use strict';
    var DialogFileDropView = Backbone.View.extend({
        className: 'dropzone dropzone-border dialog field',
        template: _.template(tmpl),
        overContainer: false,
        overDropZone: false,
        localDrag: false,
        draggingFile: null,
        enabled: false,
        editField: '',
        onDialog: true,
        events: {
            'drop': 'dropOnDropZone'
        },
        initialize: function(options) {
            if(options.enabled) {
                this.enabled = options.enabled;
            }
            if(options.editField) {
                this.editField = options.editField;
            }
            this.$el.append(this.template());

            this.$doc = $(document);
            this.$el.on('dragover', _.bind(this.dragOverOnDropZone, this));
            this.$el.on('dragleave', _.bind(this.dragLeaveFromDropZone, this));
            this.$doc.on( 'dragstart dragend drop', _.bind(function( event ) {
                this.localDrag = event.type === 'dragstart';
                iuu.log('local: ' + this.localDrag);
            }, this));

            Backbone.on('dialog:open', this.updateSize, this);
            Backbone.on('dropped', this.dropped, this);

            if(MultiUploadSettings.permission_denied) {
                this.disable();
            }

            Backbone.on('dialog:resize', this.updateSize, this);
        },
        render: function() {
            return this;
        },
        resize: function () {
        },
        updateSize: function() {
            var margin = 20;
            var h = this.$el.parent().parent().height();
            var headerH = $('#upload-file-header').outerHeight(true);
            h -= headerH;
            var footerH = $('.upload-file .modal-footer').outerHeight(true);
            h -= footerH;
            h -= margin * 2;

            var offset = this.$el.offset();
            offset.top += headerH;

            var paddingSize = parseInt(this.$el.parent().css('padding-top'));
            h -= paddingSize * 2;

            var w = this.$el.parent().width();
            this.$el.outerHeight(h);
            this.$el.outerWidth(w);
        },
        enable: function() {
            this.enabled = true;
            this.draggingFile = null;
            this.overContainer = false;
            this.overDropZone = false;
            iuu.log('enable');
        },
        disable: function() {
            this.enabled = false;
            this.draggingFile = null;
            this.overContainer = false;
            this.overDropZone = false;
            this.$el.hide();
            iuu.log('disable');
        },
        update: function() {
            if(this.overDropZone) {
                this.$el.addClass('hover');
            }
            else {
                this.$el.removeClass('hover');
            }

            if(!this.overContainer || !this.overDropZone) {
                this.draggingFile = null;
            }
        },
        isDraggingFile: function(event) {
            if ( this.draggingFile !== null ) {
                return this.draggingFile;
            }
            if ( _.isUndefined( event.originalEvent ) || _.isUndefined( event.originalEvent.dataTransfer ) ) {
                return false;
            }
            this.draggingFile = _.indexOf( event.originalEvent.dataTransfer.types, 'Files' ) > -1 &&
                _.indexOf( event.originalEvent.dataTransfer.types, 'text/plain' ) === -1;
            return this.draggingFile;
        },
        cancelEvent: function(event) {
            event.stopPropagation();
            event.preventDefault();
        },
        dragOverOnContainer: function(event) {
            if(!this.enabled || this.localDrag || !this.isDraggingFile(event)) {
                return;
            }
            this.overContainer = true;
            this.update();
        },
        dragLeaveFromContainer: function(/* event */) {
            this.overContainer = false;
            _.delay(_.bind(this.update, this), 10);
        },
        dragOverOnDropZone: function(event) {
            if(!this.enabled || this.localDrag || !this.isDraggingFile(event)) {
                return;
            }
            this.overDropZone = true;
            this.update();
            return false;
        },
        dragLeaveFromDropZone: function(/* event */) {
            this.overDropZone = false;
            this.update();
        },
        dropOnDropZone: function(event) {
            if(!this.enabled) {
                return false;
            }
            iuu.log('dropped', this.editField);
            Backbone.trigger('dropped', this);

            this.dragLeaveFromDropZone(event);
            this.dragLeaveFromContainer(event);
            this.cancelEvent(event);

            var files = event.originalEvent.dataTransfer.files;
            iuu.log(files);
            if(files.length === 0) {
                return false;
            }

            var self = this;
            _.delay(function() {
                Backbone.trigger('fileDropped', self, files);
            }, 100);

            return false;
        },
        dropped: function(source) {
            if(source && source !== this) {
                this.dragLeaveFromDropZone();
                this.dragLeaveFromContainer();
            }
        }
    });
    return DialogFileDropView;
});


