define([
    'jquery',
    'underscore',
    'backbone',
    'multi_upload_settings'
    ],
    function ($, _, Backbone, MultiUploadSettings) {
        'use strict';
        var MTModal = Backbone.View.extend({
            el: '.mt-modal',
            events: {
                'close': 'close'
            },
            permissionDenied: false,
            initialize: function () {
                if(MultiUploadSettings.permission_denied) {
                    this.permissionDenied = true;
                    return;
                }
                this.$el.on('show.bs.modal', _.bind(this.showBsModal, this));
                this.$el.on('shown.bs.modal', _.bind(this.shownBsModal, this));
                this.$el.on('hide.bs.modal', _.bind(this.hideBsModal, this));
                this.$el.on('hidden.bs.modal', _.bind(this.hiddenBsModal, this));
            },
            render: function () {
                return this;
            },
            resize: function () {
                iuu.log('MTModal:resize');
                var height = $(window).height();
                if(this.$el.length) {
                    // this.$el.height(height - 60);
                }
                this.content().css('padding-bottom', 'unset');
                Backbone.trigger('dialog:resize');
            },
            showBsModal: function () {
                iuu.log('showBsModal: ');
                this.$el.find('.modal-dialog').css('max-width', '1024px');
            },
            shownBsModal: function () {
                iuu.log('shownBsModal: ');
                this.content().css('padding-bottom', 'unset');
                Backbone.trigger('dialog:open', this);
            },
            hideBsModal: function () {
                // this.insertDummyIframe();
                this.$el.find('.modal-dialog').css('max-width', '');
            },
            hiddenBsModal: function () {
                Backbone.trigger('dialog:close', this);
            },
            insertDummyIframe: function () {
                // NOTE: クローズ時のエラーを避けるためダミーのiframeを追加する
                if(this.content().find('iframe').length == 0) {
                    this.content().append('<iframe name="modal-0" style="display:none;">');
                }
            },
            content: function () {
                return this.$el.find('.modal-content');
            },
            append: function (elem) {
                this.content().append(elem);
            },
            empty: function () {
                this.content().empty();
            },
            open: function () {
                if(this.permissionDenied) {
                    alert('権限がありません。');
                    return;
                }

                this.insertDummyIframe();

                this.$el.find('.modal-dialog').css('width', '100%');
                this.$el.find('.modal-dialog').addClass('modal-lg');

                if(MultiUploadSettings.admin_theme_id === 'admin2023') {
                    var $bsmodal = new bootstrap.Modal(this.$el.get(0), {});
                    $bsmodal.show();
                }
                else {
                    this.$el.modal();
                }

                $(window).on('resize', _.bind(this.resize, this));
            },
            close: function (event, url, fn) {
                iuu.log('MTModal:close');
                $(window).off('resize');
                this.empty();
                this.insertDummyIframe();
                this.$el.modal('hide');
                this.empty();
                if(url) {
                    window.location.href = url;
                }
                Backbone.trigger('dialog:close', this);
                $(window).trigger('dialogDisposed');
            }
        });
        return MTModal;
    }
);

