define([
    'jquery',
    'underscore',
    'backbone',
    'text!tmpl/upload_file.html',
    'multi_upload_settings'
],
function($, _, Backbone, tmpl, MultiUploadSettings) {
    'use strict';
    var UploadFileView = Backbone.View.extend({
        className: 'upload-file-options mt-draggable sortable-item',
        template: _.template(tmpl),
        events: {
            'change select[name="site_path"]': 'changeSitePath',
            'change select[name="destination"]': 'changeDestination',
            'change input[name="custom_destination"]': 'changeCustomDestination',
            'change input[name="extra_path"]': 'changeExtraPath',
            'change input[name="label"]': 'changeLabel',
            'change input[name="thumb"]': 'changeThumb',
            'change input[name="thumb_width"]': 'changeWidth',
            'change input[name="align"]': 'changeAlign',
            'change input[name="popup"]': 'changePopup',
            'change input[name="overwrite_yes"]': 'changeOverwriteYes',
            'change select[name="resize"]': 'changeResize',
            'change input[name="resize_size"]': 'changeResizeSize',
            'change input[name="resize_width"]': 'changeResizeWidth',
            'change input[name="image_defaults"]': 'changeImageDefaults',
            'change input[name="watermark_insertion"]': 'changeWatermarkInsertion',
            'change input[name="enclose_start_tag"]': 'changeEncloseStartTag',
            'change input[name="enclose_end_tag"]': 'changeEncloseEndTag',
            'click button.close': 'close',
            'click span.label': 'ignoreClick',
            'click span.sort-handle': 'ignoreClick',
            'click span.share': 'shareOption',
            'click a.cancel-destination-custom': 'cancelDestinationCustom',
        },
        initialize: function() {
            this.listenTo(this.model, 'change:sitePath', this.updateSitePath);
            this.listenTo(this.model, 'change:middlePath', this.updateMiddlePath);
            this.listenTo(this.model, 'change:destination', this.updateDestination);
            this.listenTo(this.model, 'change:customDestination', this.updateCustomDestination);
            this.listenTo(this.model, 'change:extraPath', this.updateExtraPath);
            this.listenTo(this.model, 'change:content', this.renderImage);
            this.listenTo(this.model, 'change:resizedWidth', this.updateResizedSize);
            this.listenTo(this.model, 'change:resizedHeight', this.updateResizedSize);
            this.listenTo(this.model, 'upload:file:success', this.updateStatus);
            this.listenTo(this.model, 'upload:file:failed', this.updateStatus);
            this.listenTo(this.model, 'change:height', this.updateHeight);
            this.listenTo(this.model, 'change:width', this.updateWidth);
            this.listenTo(this.model, 'change:thumb', this.updateThumb);
            this.listenTo(this.model, 'change:align', this.updateAlign);
            this.listenTo(this.model, 'change:popup', this.updatePopup);
            this.listenTo(this.model, 'change:overwriteYes', this.updateOverwrite);
            this.listenTo(this.model, 'change:imageHeight', this.updateImageHeight);
            this.listenTo(this.model, 'change:imageWidth', this.updateImageWidth);
            this.listenTo(this.model, 'change:resize', this.updateResize);
            this.listenTo(this.model, 'change:resizeSize', this.updateResizeSize);
            this.listenTo(this.model, 'change:resizeWidth', this.updateResizeWidth);
            this.listenTo(this.model, 'change:imageDefaults', this.updateImageDefaults);
            this.listenTo(this.model, 'change:watermarkInsertion', this.updateWatermarkInsertion);
            this.listenTo(this.model, 'change:encloseStartTag', this.updateEncloseStartTag);
            this.listenTo(this.model, 'change:encloseEndTag', this.updateEncloseEndTag);
            Backbone.on('update:fileOrder', this.updateFileOrder, this);
            Backbone.on('share:thumb', this.shareOptionThumb, this);
            Backbone.on('share:align', this.shareOptionAlign, this);
            Backbone.on('share:popup', this.shareOptionPopup, this);
            Backbone.on('share:overwrite', this.shareOptionOverwrite, this);
            Backbone.on('share:resize', this.shareOptionResize, this);
            Backbone.on('share:imageDefaults', this.shareOptionImageDefaults, this);
            Backbone.on('share:path', this.shareOptionPath, this);
            Backbone.on('share:watermark', this.shareOptionWatermarkInsertion, this);
            Backbone.on('share:encloseStartTag', this.shareEncloseStartTag, this);
            Backbone.on('share:encloseEndTag', this.shareEncloseEndTag, this);
        },
        syncSitePath: function() {
            var select = this.$('select[name="site_path"]');
            var value = select.val();
            var opt = this.$('option:selected', select);
            var path = opt.attr('middle_path');
            if(!path) path = '';
            this.model.attributes.sitePath = value;
            this.model.attributes.middlePath = path;
        },
        render: function() {
            var data = this.model.toJSON();
            data.cid = this.model.cid;
            data.settings = MultiUploadSettings;
            data.static_uri = StaticURI;
            this.$el.append(this.template(data));
            this.updateResizeFieldVisibility();
            this.syncSitePath();
            return this;
        },
        renderImage: function() {
            var content = this.model.get('content');
            if(content) {
                var $img = this.$('img.preview');
                $img.on('load', _.bind(this.imageLoaded, this));
                $img.hide();
                $img.attr('src', content);
            }
            return this;
        },
        close: function(event) {
            iuu.log('close option');
            event.preventDefault();
            event.stopPropagation();
            Backbone.trigger('remove:file', this.model);
            this.remove();
            this.model.cleanUp();
            this.model.destroy();
            return false;
        },
        ignoreClick: function(event) {
            iuu.log('ignoreClick');
            return false;
        },
        shareOption: function(event) {
            var target = $(event.currentTarget);
            var className = _.find(target.prop('class').split(' '), function(item) {
                if(item.match(/^share-.+$/)) {
                    return true;
                }
                return false;
            });
            var eventName = className.replace(/-/, ':');
            Backbone.trigger(eventName, this.model);
            return false;
        },
        shareOptionThumb: function(otherFile) {
            if(this.model == otherFile) {
                return;
            }
            this.model.set({
                thumb: otherFile.get('thumb'),
                width: otherFile.get('width')
            });
        },
        shareOptionAlign: function(otherFile) {
            if(this.model == otherFile) {
                return;
            }
            this.model.set({
                align: otherFile.get('align')
            });
        },
        shareOptionPopup: function(otherFile) {
            if(this.model == otherFile) {
                return;
            }
            this.model.set({
                popup: otherFile.get('popup')
            });
        },
        shareOptionOverwrite: function(otherFile) {
            this.model.set({
                overwriteYes: otherFile.get('overwriteYes')
            });
        },
        shareOptionResize: function(otherFile) {
            this.model.set({
                resize: otherFile.get('resize')
            });
            this.model.set({
                resizeSize: otherFile.get('resizeSize')
            });
            this.model.set({
                resizeWidth: otherFile.get('resizeWidth')
            });
        },
        shareOptionImageDefaults: function(otherFile) {
            if(this.model == otherFile) {
                return;
            }
            var otherFileImageDefaults = otherFile.get('imageDefaults');
            if(otherFileImageDefaults) {
                this.model.set({
                    imageDefaults: false
                });
            }
        },
        shareOptionPath: function(otherFile) {
            if(this.model == otherFile) {
                return;
            }
            // FIXME: 値のセットに順番が意味を持ってしまっているのでどうにかしろ。
            // custom destination に切り替え時にextraPathをcopyするので先に変更しておく。
            this.model.set({
                extraPath: otherFile.get('extraPath')
            });
            this.model.set({
                sitePath: otherFile.get('sitePath'),
                middlePath: otherFile.get('middlePath'),
                destination: otherFile.get('destination')
            });
            this.model.set({
                customDestination: otherFile.get('customDestination')
            });
        },
        shareOptionWatermarkInsertion: function(otherFile) {
            this.model.set({
                watermarkInsertion: otherFile.get('watermarkInsertion')
            });
        },
        shareEncloseStartTag: function(otherFile) {
            if(this.model == otherFile) {
                return;
            }
            this.model.set({
                encloseStartTag: otherFile.get('encloseStartTag'),
            });
        },
        shareEncloseEndTag: function(otherFile) {
            if(this.model == otherFile) {
                return;
            }
            this.model.set({
                encloseEndTag: otherFile.get('encloseEndTag'),
            });
        },
        imageLoaded: function() {
            _.delay(_.bind(this._imageLoaded, this), 200);
        },
        _imageLoaded: function() {
            var $img = this.$('img.preview');
            var width = $img.prop('naturalWidth'),
                height = $img.prop('naturalHeight');
            var size = $img.parent().width();
           if(width > size || height > size) {
                if(width >= height) {
                    $img.width(size);
                }
                else {
                    $img.height(size);
                }
            }
            $img.show();
            this.model.set({
                imageWidth: width,
                imageHeight: height
            });
        },
        changeSitePath: function(event) {
            var target = $(event.currentTarget);
            iuu.log('sitePath: ' + target.val());
            this.model.set({ sitePath: target.val() });
            var opt = this.$('option:selected', target);
            var path = opt.attr('middle_path');
            if(!path) path = '';
            this.model.set({ middlePath: path });
            iuu.log(this.model);
        },
        _enableCustomDestination: function(target) {
            if(!target) return;
            // hide select menu
            target.addClass('hidden');
            target.attr('disabled', 'disabled');
            // show custom destination field
            var field = $('#custom_destination-'+this.model.cid);
            field.removeClass('hidden');
            field.removeAttr('disabled');
            // copy first selection to extra path field
            field.val(target.children(':first').val());
            // hide extra path field
            var span = $('#upload-extra-path-'+this.model.cid);
            span.addClass('hidden');
            // prepend extra path to custom destination
            var extra_path = $('#extra_path-'+this.model.cid);
            if(extra_path.val() != '') {
                field.val(field.val() + '/' + extra_path.val());
                extra_path.val('');
            }
            // show cancel link
            var cancel = $('#cancel_destination_custom-'+this.model.cid);
            cancel.removeClass('hidden');
            // set initial custom destination value
            this.model.set({ customDestination: field.val() });
        },
        _disableCustomDestination: function(target) {
            if(!target) return;
            // show select menu
            target.removeClass('hidden');
            target.removeAttr('disabled');
            target.children(':first').attr('selected', 'selected');
            // hide custom destination field
            var field = $('#custom_destination-'+this.model.cid);
            field.addClass('hidden');
            field.attr('disabled', 'disabled');
            field.val('');
            // show extra path field
            var span = $('#upload-extra-path-'+this.model.cid);
            span.removeClass('hidden');
            // hide cancel link
            var cancel = $('#cancel_destination_custom-'+this.model.cid);
            cancel.addClass('hidden');
            // clear custom destination value
            this.model.set({ customDestination: '' });
        },
        changeDestination: function(event) {
            var target = $(event.currentTarget);
            iuu.log('destination: ' + target.val());
            this.model.set({ destination: target.val() });
            iuu.log(this.model);
        },
        changeCustomDestination: function(event) {
            var target = $(event.currentTarget);
            iuu.log('custom destination: ' + target.val());
            this.model.set({ customDestination: target.val() });
            iuu.log(this.model);
        },
        changeExtraPath: function(event) {
            var target = $(event.currentTarget);
            iuu.log('extraPath: ' + target.val());
            this.model.set({ extraPath: target.val() });
            iuu.log(this.model);
        },
        changeLabel: function(event) {
            var target = $(event.currentTarget);
            iuu.log('label: ' + target.val());
            this.model.set({ label: target.val() });
            iuu.log(this.model);
        },
        changeThumb: function(event) {
            var target = $(event.currentTarget);
            iuu.log('thumb: ' + target.prop('checked'));
            this.model.set({ thumb: target.prop('checked') });
            iuu.log(this.model);
        },
        changeWidth: function(event) {
            var target = $(event.currentTarget);
            iuu.log('width: ' + target.val());
            this.model.set({ width: target.val() });
            iuu.log(this.model);
        },
        changeAlign: function(event) {
            var target = $(event.currentTarget);
            iuu.log('changeAlign: ' + target.val());
            this.model.set({ align: target.val() });
            iuu.log(this.model);
        },
        changePopup: function(event) {
            var target = $(event.currentTarget);
            iuu.log('popup: ' + target.prop('checked'));
            this.model.set({ popup: target.prop('checked') });
            iuu.log(this.model);
        },
        changeOverwriteYes: function(event) {
            var target = $(event.currentTarget);
            iuu.log('overwrite_yes: ' + target.prop('checked'));
            this.model.set({ overwriteYes: target.prop('checked') });
            iuu.log(this.model);
        },
        changeResize: function(event) {
            var target = $(event.currentTarget);
            this.model.set({ resize: target.val() });
            this.updateResizeFieldVisibility();
        },
        changeResizeSize: function(event) {
            var target = $(event.currentTarget);
            this.model.set({ resizeSize: target.val() });
        },
        changeResizeWidth: function(event) {
            var target = $(event.currentTarget);
            this.model.set({ resizeWidth: target.val() });
        },
        changeImageDefaults: function(event) {
            var target = $(event.currentTarget);
            iuu.log('image_defaults: ' + target.prop('checked'));
            this.model.set({ imageDefaults: target.prop('checked') });
            iuu.log(this.model);
            Backbone.trigger('share:imageDefaults', this.model);
        },
        changeWatermarkInsertion: function(event) {
            var target = $(event.currentTarget);
            iuu.log('watermark_insertion: ' + target.prop('checked'));
            this.model.set({ watermarkInsertion: target.prop('checked') });
            iuu.log(this.model);
        },
        changeEncloseStartTag: function(event) {
            var target = $(event.currentTarget);
            this.model.set({ encloseStartTag: target.val() || '' });
        },
        changeEncloseEndTag: function(event) {
            var target = $(event.currentTarget);
            this.model.set({ encloseEndTag: target.val() || '' });
        },
        updateResizeFieldVisibility: function() {
            var value = this.$('select[name="resize"]').val();
            if(value == 0) {
                this.$('input[name="resize_size"]').hide();
                this.$('input[name="resize_width"]').hide();
            }
            else if(value == 1) {
                this.$('input[name="resize_size"]').show();
                this.$('input[name="resize_width"]').hide();
            }
            else if(value == 2) {
                this.$('input[name="resize_size"]').hide();
                this.$('input[name="resize_width"]').show();
            }
        },
        updateResizedSize: function() {
            iuu.log('updateResizedSize');
            this.$('input[name="thumb_width"]').val(this.model.get('resizedWidth'));
        },
        updateFileOrder: function() {
            var index = $('div.sortable-item').index(this.$el);
            this.model.set({ index: index }, { silent: true });
            iuu.log(this.model.cid + ' index: ' + index);
        },
        updateStatus: function(file) {
            if(file.lastError) {
                this.$('.msg-error').text(file.lastError);
                this.$('.msg-error').show();
            }
            else {
                this.$('.msg-success').show();
            }
            // アップロード後、設定を無効にする
            this.$('input').prop('disabled', true);
            this.$('span.share').hide();
            this.$('button.close').hide();
        },
        updateThumb: function() {
            var input = this.$('input[name="thumb"]');
            var value = this.model.get('thumb');
            if(!!input.prop('checked') != !!value) {
                input.prop('checked', value);
            }
        },
        updateWidth: function() {
            var input = this.$('input[name="thumb_width"]');
            var value = this.model.get('width');
            if(input.val() != value) {
                input.val(value);
            }
        },
        updateHeight: function() {
            var input = this.$('input[name="thumb_height"]');
            var value = this.model.get('height');
            if(input.val() != value) {
                input.val(value);
            }
        },
        updateAlign: function() {
            var input = this.$('input[name="align"]:checked');
            var value = this.model.get('align');
            if(input.val() != value) {
                this.$('input[name="align"][value="' + value + '"]').prop('checked', true);
            }
        },
        updatePopup: function() {
            var input = this.$('input[name="popup"]');
            var value = this.model.get('popup');
            if(!!input.prop('checked') != !!value) {
                input.prop('checked', value);
            }
        },
        updateOverwrite: function() {
            var input = this.$('input[name="overwrite_yes"]');
            var value = this.model.get('overwriteYes');
            if(!!input.prop('checked') != !!value) {
                input.prop('checked', value);
            }
        },
        updateResize: function() {
            var select = this.$('select[name="resize"]');
            var value = this.model.get('resize');
            if(select.val() != value) {
                select.val(value);
                this.updateResizeFieldVisibility();
            }
        },
        updateResizeSize: function() {
            var input = this.$('input[name="resize_size"]');
            var value = this.model.get('resizeSize');
            if(input.val() != value) {
                input.val(value);
            }
        },
        updateResizeWidth: function() {
            var input = this.$('input[name="resize_width"]');
            var value = this.model.get('resizeWidth');
            if(input.val() != value) {
                input.val(value);
            }
        },
        updateImageWidth: function() {
            this.$('span.width').text(this.model.get('imageWidth'));
        },
        updateImageHeight: function() {
            this.$('span.height').text(this.model.get('imageHeight'));
        },
        updateImageDefaults: function() {
            var input = this.$('input[name="image_defaults"]');
            var value = this.model.get('imageDefaults');
            if(!!input.prop('checked') != !!value) {
                input.prop('checked', value);
            }
        },
        updateSitePath: function() {
            var select = this.$('select[name="site_path"]');
            var value = this.model.get('sitePath');
            if(select.val() != value) {
                select.val(value);
            }
        },
        updateMiddlePath: function() {
            var input = this.$('input[name="middle_path"]');
            var value = this.model.get('middlePath');
            if(input.val() != value) {
                input.val(value);
                var select = this.$('select[name="site_path"]');
                var opt = this.$('option[middle_path="'+value+'"]');
                if(opt.length && opt.index != select[0].selectedIndex) {
                    opt.prop('selected', true);
                }
            }
        },
        updateDestination: function() {
            var select = this.$('select[name="destination"]');
            var value = this.model.get('destination');
            var prevValue = this.model.previous('destination');
            if(value == '') {
                this._enableCustomDestination(select);
            }
            else if(prevValue == '') {
                this._disableCustomDestination(select);
            }
            if(select.val() != value) {
                select.val(value);
            }
        },
        updateCustomDestination: function() {
            var input = this.$('input[name="custom_destination"]');
            var value = this.model.get('customDestination');
            if(input.val() != value) {
                input.val(value);
            }
        },
        updateExtraPath: function() {
            var input = this.$('input[name="extra_path"]');
            var value = this.model.get('extraPath');
            if(input.val() != value) {
                input.val(value);
            }
        },
        updateWatermarkInsertion: function() {
            var input = this.$('input[name="watermark_insertion"]');
            var value = this.model.get('watermarkInsertion');
            if(!!input.prop('checked') != !!value) {
                input.prop('checked', value);
            }
        },
        updateEncloseStartTag: function() {
            var input = this.$('input[name="enclose_start_tag"]');
            var value = this.model.get('encloseStartTag');
            if(input.val() != value) {
                input.val(value);
            }
        },
        updateEncloseEndTag: function() {
            var input = this.$('input[name="enclose_end_tag"]');
            var value = this.model.get('encloseEndTag');
            if(input.val() != value) {
                input.val(value);
            }
        },
        cancelDestinationCustom: function() {
            iuu.log('click: cancelDestinationCustom');
            var target = $('#destination-'+this.model.cid);
            if(target) {
                this._disableCustomDestination(target);
                this.model.set({ destination: target.val() });
            }
            return false;
        }
    });
    return UploadFileView;
});

