define([
    'jquery',
    'underscore',
    'backbone',
    'view/upload_file_collection_view',
    'text!tmpl/upload.html',
    'view/upload_progress',
    'view/dialog_file_drop_view',
    'model/upload_file',
    'multi_upload_settings'
],
function($, _, Backbone, UploadFileCollectionView, tmpl, UploadProgress, DialogFileDropView, UploadFile, MultiUploadSettings) {
    'use strict';
    var UploadView = Backbone.View.extend({
        id: 'upload-file',
        className: 'upload-file dialog dialog-screen',
        template: _.template(tmpl),
        events: {
            'click button.finish': 'finish',
            'click button.upload': 'upload',
            'click button.cancel': 'cancel'
        },
        finished: false,
        droppable: false,
        editField: '',
        closing: false,
        initialize: function(options) {
            if(options.droppable) {
                this.droppable = options.droppable;
            }
            if(options.editField) {
                this.editField = options.editField;
            }
            this.finished = false;
            $(window).on('resize', _.bind(this.resize, this));
            Backbone.on('remove:file', function() {
                if(this.model.length === 1) {
                    this.close();
                }
            }, this);
            Backbone.on('fileDropped', this.fileDropped, this);
            // 追加のdropを禁止する
            $(document).on('dragover.upload_view drop.upload_view', function() {
                return false;
            });
            // ajaxq のqueueが空になった時に送信されるイベント
            $(window).on('ajaxq:empty', _.bind(this.uploadFinished, this));
            // リサイズ
            Backbone.on('dialog:resize', this.resize, this);
            //
            $('.mt-modal').on('hide.bs.modal.uploadview', _.bind(this.cancelDialog, this));
            $('.mt-modal').on('hidden.bs.modal.uploadview', _.bind(this.close, this));
            this.closing = false;
        },
        render: function() {
            this.collectionView = new UploadFileCollectionView({ model: this.model });
            this.$el.append(this.template(this));
            this.$el.find('.modal-body').append(this.collectionView.render().$el);
            if(this.droppable) {
                this.dropView = new DialogFileDropView({
                    enabled: true,
                    editField: 'text-field'
                });
                this.$el.find('.modal-body').append(this.dropView.$el);
            }
            if(this.model.length === 0) {
                this.$('button.upload').hide();
            }
            this.resize();
            return this;
        },
        resize: function() {
            var height = $(window).height();
            if(this.$el.length) {
                this.$el.height(height - 60);
            }
        },
        close: function() {
            iuu.log('UploadView: close', this);
            this.closing = true;
            // 追加のdrop禁止を解除する
            $(document).off('dragover.upload_view drop.upload_view');
            Backbone.trigger('close:dialog');
            this.dropView = null;
            this.collectionView = null;
            this.model.reset();
            this.model.cleanUp();
            $('.mt-modal').off('hide.bs.modal.uploadview');
            $('.mt-modal').off('hidden.bs.modal.uploadview');
            this.remove();
            return;
        },
        cancelDialog: function (e) {
            iuu.log('UploadView:cancelDialog', this);
            if(!this.closing && this.model.length > 0 && !confirm('本当にキャンセルしますか？')) {
                e.preventDefault();
                e.stopImmediatePropagation();
                this.closing = false;
                return false;
            }
            this.closing = true;
        },
        cancel: function() {
            if(this.model.length > 0 && !confirm('本当にキャンセルしますか？')) {
                return false;
            }
            iuu.log('cancel');
            this.close();
            return true;
        },
        finish: function() {
            var html = '';
            var field = '';
            var preview = '';
            var assetListItems = [];
            this.model.each(function(file) {
                iuu.log('lastResult: ' + file.lastResult);
                if(file.lastResult) {
                    var content = file.lastResult.content;
                    field = file.lastResult.edit_field;
                    if(field.match(/^(customfield_|asset-field-)/)) {
                        html += content;
                    }
                    else {
                        html += file.get('encloseStartTag') + content + file.get('encloseEndTag');
                    }
                    preview = file.lastResult.preview;
                    assetListItems.push({
                        id: file.lastResult.asset_id,
                        listItem: file.lastResult.asset_list_item
                    });
                }
            });

            if(html.length > 0) {
                var append = false;
                if(field.match(/^customfield_/)) {
                    window.insertCustomFieldAsset(html, field, preview);
                    append = true;
                }
                else if(field.match(/^asset-field-/)) {
                    // Image content field
                    if(!$('#'+field).data('mt-multiple')) {
                        $('#'+field).children('.mt-draggable').remove();
                    }
                    var alert = $('#'+field).children('.alert');
                    alert.before(html);
                }
                else {
                    // HACK: 
                    // tinymceによって、Enclose Tag(例えばfigureなど)の内側にカーソルが置かれてしまうので、
                    // tinymceのリッチエディタモードの時だけ、無理やりカーソルを最後の画像の後に置く。
                    if(this.isRichEditor()) {
                        html += '<p>{$caret}</p>';
                    }
                    window.app.insertHTML(html, field);
                    append = true;
                }

                Backbone.trigger('file:inserted');

                if(append) {
                    this.appendListItem(assetListItems);
                }
            }

            iuu.log('finish');
            this.close();
            return;
        },
        isRichEditor: function() {
            if(window.app.editor.currentEditor.editor !== window.app.editor.currentEditor.source) {
                return true;
            }
            return false;
        },
        appendListItem: function(list) {
            $('#empty-asset-list').empty();
            var $assetList = $('#asset-list');
            if($assetList.length === 0) {
                return;
            }
            _.each(list, function(item) {
                if($('#list-asset-' + item.id).length > 0) {
                    return;
                }
                var asset_ids = $('#include_asset_ids').val();
                if(asset_ids.length > 0) {
                    asset_ids += ',';
                }
                $('#include_asset_ids').val(asset_ids + item.id);
                $assetList.append(item.listItem);
            }, this);
        },
        upload: function() {
            if(this.model.length === 0) {
                return this.close();
            }
            if(this.finished) {
                iuu.log('already uploaded');
                return false;
            }
            iuu.log('upload');
            this.$progress = new UploadProgress({ max: this.model.length }).render();
            this.$('button.upload').prop('disabled', true);
            this.model.each(function(file, index, list) {
                iuu.log(file.cid + ' upload submit: ');
                var promise = file.upload();
                if(index === list.length - 1) {
                    iuu.log(file.cid + ' last file');
                }
            });
        },
        uploadFinished: function() {
            iuu.log('upload finished');
            // ProgressBarを止める
            Backbone.trigger('upload:finish', self);

            this.finished = true;
            this.$('button.upload').prop('disabled', false).hide();
            this.$('button.finish').show();
            this.$('.upload-file-options-list').sortable('disable');
            this.$('.sort-handle').hide();

            this.updateImageDefaults();
        },
        getEditField: function(editField) {
            var field = '';
            if(editField === 'text-field') {
                field = window.app.editor.id.toString();
            }
            else if(editField.match(/^customfield_/)) {
                field = editField.replace(/^(.+)-field$/, '$1');
            }
            else if(match = editField.match(/^contentblock(\d+)/)) {
                field = 'editor-input-content-field-' + match[1];
            }
            return field;
        },
        // Called when files are dropped on the dialog.
        fileDropped: function(source, files) {
            iuu.log('UploadView:fileDropped:', this, this.dropView, source);
            if(this.dropView !== source) {
                return;
            }
            if(!source.onDialog) {
                return;
            }
            if(files.length === 0) {
                return;
            }
            // this.model.reset();

            // sort by file's name
            if(MultiUploadSettings.sort_files) {
                files = _.sortBy(files, 'name');
            }

            var skipFiles = 0;
            _.each(files, function(file) {
                iuu.log(file);
                if(!file.type.match(/^image\//)) {
                    iuu.log('not image, skip: ', file.type);
                    skipFiles++;
                    return;
                }
                var uploadFile = new UploadFile({
                    file: file,
                    editField: this.getEditField(source.editField)
                });
                this.model.add(uploadFile);
            }, this);

            this.collectionView.render();

            if(skipFiles > 0) {
                _.delay(function() {
                    alert('画像ファイル以外はアップロードできません。');
                }, 1000);
            }

            if(this.model.length > 0) {
                this.dropView.disable();
                this.$('button.upload').prop('disabled', false).show();
            }
        },
        updateImageDefaults: function() {
            iuu.log('[IUU] updateImageDefaults: proc');
            Backbone.ajax(
                MultiUploadSettings.mt_uri,
                {
                    data: {
                        blog_id: MultiUploadSettings.blog_id,
                        '__mode': 'iuu_image_defaults'
                    }
                }
            ).done(function(data) {
                iuu.log('[IUU] updateImageDefaults: done', data);
                MultiUploadSettings = _.extend(MultiUploadSettings, data.result);
            }).fail(function() {
                iuu.log('[IUU] updateImageDefaults: failed');
            });
        }
    });
    return UploadView;
});

